from typing import Any, List

from fastapi import APIRouter, Depends, HTTPException, Query, status
from sqlalchemy.orm import Session

from app.core.deps import get_current_active_user, get_current_superuser
from app.db.session import get_db
from app.models.user import User
from app.schemas.warehouse import Warehouse, WarehouseCreate, WarehouseUpdate
from app.services import warehouse as warehouse_service

router = APIRouter()


@router.get("/", response_model=List[Warehouse])
def read_warehouses(
    db: Session = Depends(get_db),
    skip: int = 0,
    limit: int = 100,
    active_only: bool = Query(False, description="Filter only active warehouses"),
    current_user: User = Depends(get_current_active_user),
) -> Any:
    """
    Retrieve warehouses.
    """
    warehouses = warehouse_service.get_multi(
        db, skip=skip, limit=limit, active_only=active_only
    )
    return warehouses


@router.post("/", response_model=Warehouse)
def create_warehouse(
    *,
    db: Session = Depends(get_db),
    warehouse_in: WarehouseCreate,
    current_user: User = Depends(get_current_superuser),
) -> Any:
    """
    Create new warehouse. Requires superuser access.
    """
    warehouse = warehouse_service.get_by_code(db, code=warehouse_in.code)
    if warehouse:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="A warehouse with this code already exists",
        )
    warehouse = warehouse_service.create(db, obj_in=warehouse_in)
    return warehouse


@router.get("/{warehouse_id}", response_model=Warehouse)
def read_warehouse(
    *,
    db: Session = Depends(get_db),
    warehouse_id: int,
    current_user: User = Depends(get_current_active_user),
) -> Any:
    """
    Get warehouse by ID.
    """
    warehouse = warehouse_service.get(db, warehouse_id=warehouse_id)
    if not warehouse:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Warehouse not found",
        )
    return warehouse


@router.put("/{warehouse_id}", response_model=Warehouse)
def update_warehouse(
    *,
    db: Session = Depends(get_db),
    warehouse_id: int,
    warehouse_in: WarehouseUpdate,
    current_user: User = Depends(get_current_superuser),
) -> Any:
    """
    Update a warehouse. Requires superuser access.
    """
    warehouse = warehouse_service.get(db, warehouse_id=warehouse_id)
    if not warehouse:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Warehouse not found",
        )
    warehouse = warehouse_service.update(db, db_obj=warehouse, obj_in=warehouse_in)
    return warehouse


@router.delete("/{warehouse_id}", response_model=Warehouse)
def delete_warehouse(
    *,
    db: Session = Depends(get_db),
    warehouse_id: int,
    current_user: User = Depends(get_current_superuser),
) -> Any:
    """
    Delete a warehouse. Requires superuser access.
    """
    warehouse = warehouse_service.get(db, warehouse_id=warehouse_id)
    if not warehouse:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Warehouse not found",
        )
    warehouse = warehouse_service.remove(db, warehouse_id=warehouse_id)
    return warehouse